import io
import pandas as pd
from .tokenization import FullTokenizer, load_vocab
import tqdm
import numpy as np

def load_vectors(fname, type='index',vocab=None):
    print('Loading vectors from ', fname,' type: ',type)
    fin = io.open(fname, 'r', encoding='utf-8', newline='\n', errors='ignore')
    n, d = map(int, fin.readline().split())
    data = {}
    i=1
    for line in fin:
        tokens = line.rstrip().split(' ')
        if type=='index':
            data[tokens[0]] = i
            i+=1
        else:
            if tokens[0] in vocab:
                data[tokens[0]] = list(map(float, tokens[1:]))
        
    return data

class Processor():
    def __init__(self,vector_file, input_file, column_name):
        self.vector_file = vector_file
        self.input_file = input_file
        self.column_name = column_name


    def process(self, output_dir, special_tokens=[]):

        vectors_dict = load_vectors(self.vector_file)
        df = pd.read_csv(self.input_file)

        full_vocab_file = output_dir+'/full_vocab.txt'
        full_vocab = list(vectors_dict.keys())
        full_vocab.extend(special_tokens)
        
        self.__write(full_vocab, full_vocab_file)

        full_vocab_tokenizer = FullTokenizer(
            vocab_file=full_vocab_file, do_lower_case=True)
        
        unique_subword = self.__get_unique(full_vocab_tokenizer, df[self.column_name].values)

        vocab_file = output_dir+'/vocab.txt'
        self.__write(unique_subword, vocab_file)

        new_vocab = load_vocab(vocab_file)
        new_vectors_dict = load_vectors(self.vector_file, 'embedding', new_vocab)

        final_vocab = list(new_vectors_dict.keys())
        final_vocab.extend(special_tokens)

        self.__write(final_vocab, vocab_file)
        embedding_matrix = self.__get_final_embedding(list(new_vectors_dict.values()), special_tokens)

        embedding_file = output_dir+'/embeddings.npy'
        self.__write_final_embedding(embedding_matrix, embedding_file)

        print('Processing Done !')
        print('Vocab stored at :',vocab_file ,' of size: ',len(final_vocab))
        print('Embedding stored at :', embedding_file, 'of shape: ', embedding_matrix.shape)

    def __get_final_embedding(self, embedding_list, special_tokens):
        print('Making Final Embedding ...')
        for t in special_tokens:
            embedding_list.append(list(np.random.uniform(-1,1,300)))

        embedding_matrix = np.array(embedding_list)
        return embedding_matrix

    def __write_final_embedding(self, embedding_matrix, embedding_file):
        print('Writing embedding at ', embedding_file)
        np.save(embedding_file, embedding_matrix)


    def __get_unique(self, tokenizer, text_list):
        print('Generating unique tokens ...')
        unique_subword = set()
        unknown_count = 0
        for i in tqdm.tqdm(range(len(text_list))):
            subword_list = tokenizer.tokenize(str(text_list[i]))
            
            for subword in subword_list:
                if subword=='[UNK]':
                    unknown_count+=1
                unique_subword.add(subword)
        
        print('No of unique subwords :', len(list(unique_subword)))
        print('No of Unknowns:', unknown_count)
        return list(unique_subword)

    def __write(self, text_list, file_name):
        print('Writing vocab at ', file_name)
        with open(file_name, 'w') as file:
            for key in text_list:
                file.write(key)
                file.write('\n')


from tensorflow.keras.preprocessing.sequence import pad_sequences
import spacy
import deepdish as dd
from tqdm import tqdm
import os


# GLobal Variable
nlp = spacy.load('en_core_web_sm', disable=['parser','ner','tagger'])
nlp.vocab.add_flag(lambda s: s.lower() in spacy.lang.en.stop_words.STOP_WORDS, spacy.attrs.IS_STOP)

def get_word_sequences(text_list, token_file_path=None, word_dict={}, lemma_dict={}, word_index=1, max_length = 250):
    
    if(token_file_path is not None) and os.path.exists(token_file_path):
        token_data = dd.io.load(token_file_path)
        word_dict = token_data['word_dict']
        lemma_dict = token_data['lemma_dict']
        word_index = token_data['word_index']
        
    previous_word_index = word_index
    docs = nlp.pipe(text_list, n_threads = 4)
    word_sequences = []

    for doc in tqdm(docs):
        word_seq = []
        for token in doc:
            if (token.text not in word_dict) and (token.pos_ is not "PUNCT"):
                word_dict[token.text] = word_index
                word_index += 1
                lemma_dict[token.text] = token.lemma_
            if token.pos_ is not "PUNCT":
                word_seq.append(word_dict[token.text])
        word_sequences.append(word_seq)
    del docs
    
    if previous_word_index!=word_index and token_file_path!=None:
        print('Changes Saved at ', token_file_path)
        data = {'word_dict': word_dict, 'lemma_dict':lemma_dict, 'word_index':word_index}
        dd.io.save(token_file_path, data)
    
    return pad_sequences(word_sequences, max_length)

def get_tokenids_start_end_position(tokenizer, text, extracted_text,extra_text=None, verbose=False):

    text = '[CLS] '+text+' [SEP]'
    if extra_text:
        text = text+' '+extra_text+' [SEP]'
    index = text.find(extracted_text)
    
    first = tokenizer.tokenize(text[0:index])
    middle = tokenizer.tokenize(text[index:index+len(extracted_text)])
    last = tokenizer.tokenize(text[index+len(extracted_text):len(text)])
    
    if verbose:
        print(text[0:index], text[index:index+len(extracted_text)], text[index+len(extracted_text):len(text)])
    
    tokens = first+middle+last
    token_ids = tokenizer.convert_tokens_to_ids(tokens)
    start = len(first)
    end = len(first)+len(middle)
    
    return token_ids, start, end


from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import OneHotEncoder
import numpy as np

class Dataset():

    def __init__(self, data_config):
        self.data_config = self.__get_default_data_config()
        self.__temp_storage = {}

        for key, value in data_config.items():
            self.data_config[key] = value


    def __get_default_data_config(self):
        data_config = {
                        'data_class':'multi-class',
                        'x_columns':[],
                        'y_columns':[],
                        'one_hot_encoded_columns':[],
                        'label_encoded_columns':[],
                        'data':None,
                        'split_ratio':0.2,
                        'random_state':3107
                      }
        return data_config

    def __label_encode(self, data, name):
        name = name + "_factor"

        if name in self.__temp_storage:
            label_encoder = self.__temp_storage[name]
            encoded = label_encoder.fit_transform(data)
        else:
            label_encoder = LabelEncoder()
            self.__temp_storage[name] = label_encoder
            encoded = label_encoder.fit_transform(data)
            self.data_config[name] = np.array(label_encoder.classes_, dtype=object)

        return encoded

    def __one_hot_encode(self, data, name):
        
        name = name + "_factor"
        data = data.tolist()
        data = [[d] for d in data]

        if name in self.__temp_storage:
            onehot_encoder = self.__temp_storage[name]
            encoded = onehot_encoder.transform(data)
        else:
            onehot_encoder = OneHotEncoder(handle_unknown='ignore')
            onehot_encoder.fit(data)
            self.__temp_storage[name] = onehot_encoder
            encoded = onehot_encoder.transform(data)
            self.data_config[name] = np.array(onehot_encoder.categories_[0], dtype=object)
        return encoded.toarray()

    def __get_processed_data(self, data, type):

        result = {}
        for column in self.data_config[type]:

            if column in self.data_config['one_hot_encoded_columns']:
                result[column] = np.array(self.__one_hot_encode(data[column].values, column))
            elif column in self.data_config['label_encoded_columns']:
                result[column] = np.array(self.__label_encode(data[column].values, column))
            else:
                result[column] = np.array(data[column].values)
        return result
    

    def get_train_test_data(self):

        train = {}
        test = {}

        if self.data_config['data_class'] == 'multi-class':
            train_lines, test_lines, train_labels, test_labels = train_test_split(self.data_config['data'][self.data_config['x_columns']],
                                                                self.data_config['data'][self.data_config['y_columns']], 
                                                                test_size=self.data_config['split_ratio'], 
                                                                random_state=self.data_config['random_state'],
                                                                stratify=self.data_config['data'][self.data_config['y_columns']])
            X_train = self.__get_processed_data(train_lines,'x_columns')
            Y_train = self.__get_processed_data(train_labels,'y_columns')
            X_test = self.__get_processed_data(test_lines,'x_columns')
            Y_test = self.__get_processed_data(test_labels,'y_columns')

            train['X_train'] = X_train
            train['Y_train'] = Y_train
            test['X_test'] = X_test
            test['Y_test'] = Y_test
            
        elif self.data_config['data_class'] == 'multi-label':
            train_lines, test_lines, train_labels, test_labels = train_test_split(self.data_config['data'][self.data_config['x_columns']],
                                                                self.data_config['data'][self.data_config['y_columns']], 
                                                                test_size=self.data_config['split_ratio'], 
                                                                random_state=self.data_config['random_state'])

            X_train = self.__get_processed_data(train_lines,'x_columns')
            Y_train = np.array(train_labels)
            X_test = self.__get_processed_data(test_lines,'x_columns')
            Y_test = np.array(test_labels)

            train['X_train'] = X_train
            train['Y_train'] = Y_train
            test['X_test'] = X_test
            test['Y_test'] = Y_test
            
        else:
            print('Developing')
        
        return train, test


# General imports
import os, sys, re, unicodedata

# custom imports
from gensim.utils import deaccent
from collections import Counter
import argparse

### Initial State
verbose = True
global_lower=True

WPLACEHOLDER = 'word_placeholder'

def _check_replace(w):
    return not bool(re.search(WPLACEHOLDER, w))

def _make_cleaning(s, c_dict):
    if _check_replace(s):
        s = s.translate(c_dict)
    return s

def _check_vocab(c_list, vocabulary, response='default'):
    try:
        words = set([w for line in c_list for w in line.split()])
        print('Total Words :',len(words))
        u_list = words.difference(set(vocabulary))
        k_list = words.difference(u_list)
    
        if response=='default':
            print('Unknown words:', len(u_list), '| Known words:', len(k_list))
        elif response=='unknown_list':
            return list(u_list)
        elif response=='known_list':
            return list(k_list)
    except:
        return []
    
def _make_dict_cleaning(s, w_dict):
    if _check_replace(s):
        s = w_dict.get(s, s)
    return s

def _print_dict(temp_dict, n_items=10):
    run = 0
    for k,v in temp_dict.items():
        print(k,'---',v)
        run +=1
        if run==n_items:
            break  


#################### Main Function #################

def to_lower(data):
    if verbose: print('#'*10 ,'Step - Lowering everything:')
    data = list(map(lambda x: x.lower(), data))
    return data

def to_normalize(data):
    if verbose: print('#'*10 ,'Step - Normalize chars and dots:')

    normalized_chars = {}
    
    chars = '‒–―‐—━—-▬'
    for char in chars:
        normalized_chars[ord(char)] = '-'

    chars = '«»“”¨"'
    for char in chars:
        normalized_chars[ord(char)] = '"'

    chars = "’'ʻˈ´`′‘’\x92"
    for char in chars:
        normalized_chars[ord(char)] = "'"

    chars = '̲_'
    for char in chars:
        normalized_chars[ord(char)] = '_'

    chars = '\xad\x7f'
    for char in chars:
        normalized_chars[ord(char)] = ''

    chars = '\n\r\t\u200b\x96'
    for char in chars:
        normalized_chars[ord(char)] = ' '

    # Normalize chars and dots - SEE HELPER FOR DETAILS
    # Global
    data = list(map(lambda x: ' '.join([_make_cleaning(i,normalized_chars) for i in x.split()]), data))
    data = list(map(lambda x: re.sub('\(dot\)', '.', x), data))
    data = list(map(lambda x: deaccent(x), data))
    
    return data

def remove_control_char(data):
    if verbose: print('#'*10 ,'Step - Control Chars:')
    global_chars_list = list(set([c for line in data for c in line]))
    chars_dict = {c:'' for c in global_chars_list if unicodedata.category(c)[0]=='C'}
    data = list(map(lambda x: ' '.join([_make_cleaning(i,chars_dict) for i in x.split()]), data))
    
    return data

def remove_href(data):
    if verbose: print('#'*10 ,'Step - Remove hrefs:')
    data = list(map(lambda x: re.sub(re.findall(r'\<a(.*?)\>', x)[0], '', x) if (len(re.findall(r'\<a (.*?)\>', x))>0) and ('href' in re.findall(r'\<a (.*?)\>', x)[0]) else x, data))
    return data

def remove_duplicate(data):
    # Duplicated dots, question marks and exclamations
    # Locallocal_vocab
    if verbose: print('#' * 10, 'Step - Duplicated Chars:')
    local_vocab={}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_dict = {}
    
    for word in temp_vocab:
        new_word = word
        if (Counter(word)['.']>1) or (Counter(word)['!']>1) or (Counter(word)['?']>1) or (Counter(word)[',']>1):
            if (Counter(word)['.']>1):
                new_word = re.sub('\.\.+', ' . . . ', new_word)
            if (Counter(word)['!']>1):
                new_word = re.sub('\!\!+', ' ! ! ! ', new_word)
            if (Counter(word)['?']>1):
                new_word = re.sub('\?\?+', ' ? ? ? ', new_word)
            if (Counter(word)[',']>1):
                new_word = re.sub('\,\,+', ' , , , ', new_word)
            temp_dict[word] = new_word
            
    temp_dict = {k: v for k, v in temp_dict.items() if k != v}
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    return data

def remove_underscore(data):
    if verbose: print('#' * 10, 'Step - Remove underscore:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_dict = {}
    for word in temp_vocab:
        if (len(re.compile('[a-zA-Z0-9\-\.\,\/\']').sub('', word))/len(word) > 0.6) and ('_' in word):
            temp_dict[word] = re.sub('_', '', word)
    print(data[0:1])
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def seperate_spam_chars(data):
    if verbose: print('#' * 10, 'Step - Spam chars repetition:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_dict = {}
    for word in temp_vocab:
        if (len(re.compile('[a-zA-Z0-9\-\.\,\/\']').sub('', word))/len(word) > 0.6) and (len(Counter(word))==1) and (len(word)>2):
            temp_dict[word] = ' '.join([' ' + next(iter(Counter(word).keys())) + ' ' for i in range(3)])
    print(temp_dict)
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    return data

def seperate_brakets_quotes(data):
    if verbose: print('#' * 10, 'Step - Brackets and quotes:')
    chars = '()[]{}<>"'
    chars_dict = {ord(c):f' {c} ' for c in chars}
    data = list(map(lambda x: ' '.join([_make_cleaning(i,chars_dict) for i in x.split()]), data))
    return data

def break_short_words(data):
    if verbose: print('#' * 10, 'Step - Break long words:') 
    
    temp_vocab = list(set([c for line in data for c in line.split()]))
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_vocab = [k for k in temp_vocab if len(k)<=20]

    temp_dict = {}
    for word in temp_vocab:
        if '/' in word:
            temp_dict[word] = re.sub('/', ' / ', word)

    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def break_long_words(data):
    if verbose: print('#' * 10, 'Step - Break long words:') 
    
    temp_vocab = list(set([c for line in data for c in line.split()]))
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_vocab = [k for k in temp_vocab if len(k)>20]

    temp_dict = {}
    for word in temp_vocab:
        if '_' in word:
            temp_dict[word] = re.sub('_', ' ', word)
        elif '/' in word:
            temp_dict[word] = re.sub('/', ' / ', word)
        elif len(' '.join(word.split('-')).split())>2:
            temp_dict[word] = re.sub('-', ' ', word)

    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def remove_ending_underscore(data):
    if verbose: print('#' * 10, 'Step - Remove ending underscore:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if (_check_replace(k)) and ('_' in k)]
    temp_dict = {}
    for word in temp_vocab:
        new_word = word
        if word[len(word)-1]=='_':
            for i in range(len(word),0,-1):
                if word[i-1]!='_':
                    new_word = word[:i]
                    temp_dict[word] = new_word   
                    break
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def remove_starting_underscore(data):
    if verbose: print('#' * 10, 'Step - Remove starting underscore:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if (_check_replace(k)) and ('_' in k)]
    temp_dict = {}
    for word in temp_vocab:
        new_word = word
        if word[len(word)-1]=='_':
            for i in range(len(word)):
                if word[i]!='_':
                    new_word = word[:i]
                    temp_dict[word] = new_word   
                    break
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def seperate_end_word_punctuations(data):
    if verbose: print('#' * 10, 'Step - End word punctuations:')
    
    temp_vocab = list(set([c for line in data for c in line.split()]))
    temp_vocab = [k for k in temp_vocab if (_check_replace(k)) and (not k[len(k)-1].isalnum())]
    temp_dict = {}
    for word in temp_vocab:
        new_word = word
        for i in range(len(word),0,-1):
            if word[i-1].isalnum():
                new_word = word[:i] + ' ' + word[i:]
                break
        temp_dict[word] = new_word     
    temp_dict = {k: v for k, v in temp_dict.items() if k != v}
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def seperate_start_word_punctuations(data):
    if verbose: print('#' * 10, 'Step - Start word punctuations:')
    
    temp_vocab = list(set([c for line in data for c in line.split()]))
    temp_vocab = [k for k in temp_vocab if (_check_replace(k)) and (not k[0].isalnum())]
    temp_dict = {}
    for word in temp_vocab:
        new_word = word
        for i in range(len(word)):
            if word[i].isalnum():
                new_word = word[:i] + ' ' + word[i:]
                break
        temp_dict[word] = new_word     
    temp_dict = {k: v for k, v in temp_dict.items() if k != v}
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data

def clean_contractions(data):
    

    helper_contractions = {
     "aren't": 'are not',
     "Aren't": 'Are not',
     "AREN'T": 'ARE NOT',
     "C'est": "C'est",
     "C'mon": "C'mon",
     "c'mon": "c'mon",
     "can't": 'cannot',
     "Can't": 'Cannot',
     "CAN'T": 'CANNOT',
     "con't": 'continued',
     "cont'd": 'continued',
     "could've": 'could have',
     "couldn't": 'could not',
     "Couldn't": 'Could not',
     "didn't": 'did not',
     "Didn't": 'Did not',
     "DIDN'T": 'DID NOT',
     "don't": 'do not',
     "Don't": 'Do not',
     "DON'T": 'DO NOT',
     "doesn't": 'does not',
     "Doesn't": 'Does not',
     "else's": 'else',
     "gov's": 'government',
     "Gov's": 'government',
     "gov't": 'government',
     "Gov't": 'government',
     "govt's": 'government',
     "gov'ts": 'governments',
     "hadn't": 'had not',
     "hasn't": 'has not',
     "Hasn't": 'Has not',
     "haven't": 'have not',
     "Haven't": 'Have not',
     "he's": 'he is',
     "He's": 'He is',
     "he'll": 'he will',
     "He'll": 'He will',
     "he'd": 'he would',
     "He'd": 'He would',
     "Here's": 'Here is',
     "here's": 'here is',
     "I'm": 'I am',
     "i'm": 'i am',
     "I'M": 'I am',
     "I've": 'I have',
     "i've": 'i have',
     "I'll": 'I will',
     "i'll": 'i will',
     "I'd": 'I would',
     "i'd": 'i would',
     "ain't": 'is not',
     "isn't": 'is not',
     "Isn't": 'Is not',
     "ISN'T": 'IS NOT',
     "it's": 'it is',
     "It's": 'It is',
     "IT'S": 'IT IS',
     "I's": 'It is',
     "i's": 'it is',
     "it'll": 'it will',
     "It'll": 'It will',
     "it'd": 'it would',
     "It'd": 'It would',
     "Let's": "Let's",
     "let's": 'let us',
     "ma'am": 'madam',
     "Ma'am": "Madam",
     "she's": 'she is',
     "She's": 'She is',
     "she'll": 'she will',
     "She'll": 'She will',
     "she'd": 'she would',
     "She'd": 'She would',
     "shouldn't": 'should not',
     "that's": 'that is',
     "That's": 'That is',
     "THAT'S": 'THAT IS',
     "THAT's": 'THAT IS',
     "that'll": 'that will',
     "That'll": 'That will',
     "there's": 'there is',
     "There's": 'There is',
     "there'll": 'there will',
     "There'll": 'There will',
     "there'd": 'there would',
     "they're": 'they are',
     "They're": 'They are',
     "they've": 'they have',
     "They've": 'They Have',
     "they'll": 'they will',
     "They'll": 'They will',
     "they'd": 'they would',
     "They'd": 'They would',
     "wasn't": 'was not',
     "we're": 'we are',
     "We're": 'We are',
     "we've": 'we have',
     "We've": 'We have',
     "we'll": 'we will',
     "We'll": 'We will',
     "we'd": 'we would',
     "We'd": 'We would',
     "What'll": 'What will',
     "weren't": 'were not',
     "Weren't": 'Were not',
     "what's": 'what is',
     "What's": 'What is',
     "When's": 'When is',
     "Where's": 'Where is',
     "where's": 'where is',
     "Where'd": 'Where would',
     "who're": 'who are',
     "who've": 'who have',
     "who's": 'who is',
     "Who's": 'Who is',
     "who'll": 'who will',
     "who'd": 'Who would',
     "Who'd": 'Who would',
     "won't": 'will not',
     "Won't": 'will not',
     "WON'T": 'WILL NOT',
     "would've": 'would have',
     "wouldn't": 'would not',
     "Wouldn't": 'Would not',
     "would't": 'would not',
     "Would't": 'Would not',
     "y'all": 'you all',
     "Y'all": 'You all',
     "you're": 'you are',
     "You're": 'You are',
     "YOU'RE": 'YOU ARE',
     "you've": 'you have',
     "You've": 'You have',
     "y'know": 'you know',
     "Y'know": 'You know',
     "ya'll": 'you will',
     "you'll": 'you will',
     "You'll": 'You will',
     "you'd": 'you would',
     "You'd": 'You would',
     "Y'got": 'You got',
     'cause': 'because',
     "had'nt": 'had not',
     "Had'nt": 'Had not',
     "how'd": 'how did',
     "how'd'y": 'how do you',
     "how'll": 'how will',
     "how's": 'how is',
     "I'd've": 'I would have',
     "I'll've": 'I will have',
     "i'd've": 'i would have',
     "i'll've": 'i will have',
     "it'd've": 'it would have',
     "it'll've": 'it will have',
     "mayn't": 'may not',
     "might've": 'might have',
     "mightn't": 'might not',
     "mightn't've": 'might not have',
     "must've": 'must have',
     "mustn't": 'must not',
     "mustn't've": 'must not have',
     "needn't": 'need not',
     "needn't've": 'need not have',
     "o'clock": 'of the clock',
     "oughtn't": 'ought not',
     "oughtn't've": 'ought not have',
     "shan't": 'shall not',
     "sha'n't": 'shall not',
     "shan't've": 'shall not have',
     "she'd've": 'she would have',
     "she'll've": 'she will have',
     "should've": 'should have',
     "shouldn't've": 'should not have',
     "so've": 'so have',
     "so's": 'so as',
     "this's": 'this is',
     "that'd": 'that would',
     "that'd've": 'that would have',
     "there'd've": 'there would have',
     "they'd've": 'they would have',
     "they'll've": 'they will have',
     "to've": 'to have',
     "we'd've": 'we would have',
     "we'll've": 'we will have',
     "what'll": 'what will',
     "what'll've": 'what will have',
     "what're": 'what are',
     "what've": 'what have',
     "when's": 'when is',
     "when've": 'when have',
     "where'd": 'where did',
     "where've": 'where have',
     "who'll've": 'who will have',
     "why's": 'why is',
     "why've": 'why have',
     "will've": 'will have',
     "won't've": 'will not have',
     "wouldn't've": 'would not have',
     "y'all'd": 'you all would',
     "y'all'd've": 'you all would have',
     "y'all're": 'you all are',
     "y'all've": 'you all have',
     "you'd've": 'you would have',
     "you'll've": 'you will have'}
    if verbose: print('#' * 10, 'Step - Contractions:')
    # Apply spellchecker for contractions
    # Local (only unknown words)
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if (_check_replace(k)) and ("'" in k)]
    temp_dict = {}
    for word in temp_vocab:
        if word in helper_contractions:
            temp_dict[word] = helper_contractions[word]
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict) 
    return data

def remove_s(data):
    if verbose: print('#' * 10, 'Step - Remove "s:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_dict = {k:k[:-2] for k in temp_vocab if (_check_replace(k)) and (k.lower()[-2:]=="'s")}
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict) 
    return data

def isolate_numbers(data):
    if verbose: print('#' * 10, 'Step - Isolate numbers:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]
    temp_dict = {}
    for word in temp_vocab:
        if re.compile('[a-zA-Z]').sub('', word) == word:
            if re.compile('[0-9]').sub('', word) != word:
                temp_dict[word] = word

    global_chars_list = list(set([c for line in temp_dict for c in line]))
    chars = ''.join([c for c in global_chars_list if not c.isdigit()])
    chars_dict = {ord(c):f' {c} ' for c in chars}                
    temp_dict = {k:_make_cleaning(k,chars_dict) for k in temp_dict}

    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
        
    return data

def regex_split_word(data):
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]

    temp_dict = {}
    for word in temp_vocab:
        if len(re.compile('[a-zA-Z0-9\*]').sub('', word))>0:
            chars = re.compile('[a-zA-Z0-9\*]').sub('', word)
            temp_dict[word] = ''.join([' ' + c + ' ' if c in chars else c for c in word])

    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    
    if verbose: _print_dict(temp_dict)
    return data

# L33T vocabulary (SLOW)
# https://simple.wikipedia.org/wiki/Leet
# Local (only unknown words)

def leet_clean(data):
    def __convert_leet(word):
        # basic conversion 
        word = re.sub('0', 'o', word)
        word = re.sub('1', 'i', word)
        word = re.sub('3', 'e', word)
        word = re.sub('\$', 's', word)
        word = re.sub('\@', 'a', word)
        return word
    if verbose: print('#' * 10, 'Step - L33T (with vocab check):')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if _check_replace(k)]

    temp_dict = {}
    for word in temp_vocab:
        new_word = __convert_leet(word)
    
        if (new_word!=word): 
            if (len(word)>2) and (new_word in local_vocab):
                temp_dict[word] = new_word

    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict) 
    return data
    

def clean_open_holded_words(data):
    if verbose: print('#' * 10, 'Step - Open Holded words:')
        
    temp_vocab = list(set([c for line in data for c in line.split()]))
    temp_vocab = [k for k in temp_vocab if (not _check_replace(k))]
    temp_dict = {}
    for word in temp_vocab:
        temp_dict[word] = re.sub('___', ' ', word[17:-1])
    data = list(map(lambda x: ' '.join([temp_dict.get(i, i) for i in x.split()]), data))
    data = list(map(lambda x: ' '.join([i for i in x.split()]), data))
    
    return data

def clean_multiple_form(data):
    if verbose: print('#' * 10, 'Step - Multiple form:')
    local_vocab = {}
    temp_vocab = _check_vocab(data, local_vocab, response='unknown_list')
    temp_vocab = [k for k in temp_vocab if (k[-1:]=='s') and (len(k)>4)]
    temp_dict = {k:k[:-1] for k in temp_vocab if (k[:-1] in local_vocab)}
    data = list(map(lambda x: ' '.join([_make_dict_cleaning(i,temp_dict) for i in x.split()]), data))
    if verbose: _print_dict(temp_dict)
    return data



#### Main Funtion ######

def clean_v1(data_list, v = True,g=True):
    global verbose, global_lower
    verbose = v
    global_lower = g
    #data_list = data.to_list()
    data_list = to_lower(data_list)
    data_list = to_normalize(data_list)
    data_list = remove_href(data_list)
    data_list = remove_control_char(data_list)
    data_list = remove_duplicate(data_list)
    data_list = remove_underscore(data_list)
    data_list = seperate_spam_chars(data_list)
    data_list = seperate_brakets_quotes(data_list)
    data_list = break_short_words(data_list)
    data_list = break_long_words(data_list)
    data_list = remove_ending_underscore(data_list)
    data_list = remove_starting_underscore(data_list)
    data_list = seperate_end_word_punctuations(data_list)
    data_list = seperate_start_word_punctuations(data_list)
    data_list = clean_contractions(data_list)
    data_list = remove_s(data_list)
    data_list = isolate_numbers(data_list)
    data_list = regex_split_word(data_list)
    data_list = leet_clean(data_list)
    data_list = clean_open_holded_words(data_list)
    data_list = clean_multiple_form(data_list)

    return data_list


def clean_v2(data_list, verbose = True,global_lower=True):
    #data_list = data.to_list()
    data_list = to_lower(data_list)
    data_list = to_normalize(data_list)
    data_list = remove_href(data_list)
    data_list = remove_control_char(data_list)
    data_list = remove_duplicate(data_list)
    data_list = remove_underscore(data_list)
    data_list = seperate_spam_chars(data_list)
    data_list = seperate_brakets_quotes(data_list)
    data_list = break_short_words(data_list)
    data_list = break_long_words(data_list)
    data_list = remove_ending_underscore(data_list)
    data_list = remove_starting_underscore(data_list)
    data_list = seperate_end_word_punctuations(data_list)
    data_list = seperate_start_word_punctuations(data_list)
    data_list = clean_contractions(data_list)
    data_list = remove_s(data_list)
    data_list = isolate_numbers(data_list)
    data_list = regex_split_word(data_list)
    data_list = leet_clean(data_list)
    data_list = clean_open_holded_words(data_list)
    data_list = clean_multiple_form(data_list)

    return data_list


if __name__=='__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument("--data_path", help="input data path")
    parser.add_argument("--output_path", help="output data path")
    parser.add_argument("--target_column", help="target column")

    DATA_PATH = args.data_path #'data/processed/MH_Investor_Data_processed'
    OUTPUT_PATH = args.output_path #DATA_PATH+'_v2'
    C_NAME = args.target_column #'Verbatim'

    data_df = pd.read_csv(DATA_PATH)
    data = data_df[C_NAME]
    data_list = data.to_list()
    data_list = clean_v1(data_list)

    C_OUTPUT_NAME = 'Cleaned_'+C_NAME
    data_df[C_OUTPUT_NAME] = data_list
    data_df.to_csv(OUTPUT_PATH)
    print('Done !')




from en_core_web_sm import load
import tqdm
import numpy as np


nlp = load()

pos_dict={'VERB': 0,
  'NOUN': 1,
  'ADP': 2,
  'SCONJ': 3,
  'SYM': 4,
  'CCONJ': 5,
  'SPACE': 6,
  'X': 7,
  'PUNCT': 8,
  'INTJ': 9,
  'PROPN': 10,
  'ADV': 11,
  'AUX': 12,
  'ADJ': 13,
  'DET': 14,
  'NUM': 15,
  'PRON': 16,
  'PART': 17}

tag_dict={'VBN': 0,
  'RBR': 1,
  'EX': 2,
  '_SP': 3,
  'VBD': 4,
  'NN': 5,
  'SYM': 6,
  'RBS': 7,
  ':': 8,
  'NNS': 9,
  'WP$': 10,
  'UH': 11,
  'JJR': 12,
  '-RRB-': 13,
  'CC': 14,
  'VBZ': 15,
  'TO': 16,
  'PRP$': 17,
  ',': 18,
  'JJ': 19,
  '-LRB-': 20,
  'RB': 21,
  'NNP': 22,
  'DT': 23,
  'RP': 24,
  'VBG': 25,
  "''": 26,
  'WDT': 27,
  'MD': 28,
  'FW': 29,
  'CD': 30,
  'PRP': 31,
  'JJS': 32,
  'VB': 33,
  'WP': 34,
  'LS': 35,
  'XX': 36,
  'AFX': 37,
  'WRB': 38,
  'HYPH': 39,
  'POS': 40,
  'VBP': 41,
  'NFP': 42,
  '``': 43,
  'NNPS': 44,
  'PDT': 45,
  '.': 46,
  'IN': 47}

dep_dict={'nsubj': 0,
  '': 1,
  'advcl': 2,
  'prep': 3,
  'relcl': 4,
  'intj': 5,
  'ccomp': 6,
  'amod': 7,
  'appos': 8,
  'prt': 9,
  'nmod': 10,
  'aux': 11,
  'xcomp': 12,
  'csubjpass': 13,
  'case': 14,
  'pobj': 15,
  'oprd': 16,
  'nummod': 17,
  'punct': 18,
  'compound': 19,
  'attr': 20,
  'advmod': 21,
  'meta': 22,
  'poss': 23,
  'parataxis': 24,
  'dative': 25,
  'cc': 26,
  'acomp': 27,
  'neg': 28,
  'dobj': 29,
  'agent': 30,
  'quantmod': 31,
  'det': 32,
  'auxpass': 33,
  'mark': 34,
  'pcomp': 35,
  'expl': 36,
  'nsubjpass': 37,
  'npadvmod': 38,
  'acl': 39,
  'conj': 40,
  'predet': 41,
  'csubj': 42,
  'dep': 43,
  'preconj': 44,
  'ROOT': 45}

def add_padding(tokens, max_len):
    
    if len(tokens)==max_len:
        return tokens
    elif len(tokens) > max_len:
        return tokens[0:max_len]
    else:
        while(len(tokens)<max_len):
            tokens.append(0)
        return tokens

def get_tokens(text):
    doc = nlp(text)
    final_token_1 = []
    final_token_2 = []
    final_token_3 = []
    for token in doc:
        final_token_1.append(pos_dict[token.pos_])
        final_token_2.append(tag_dict[token.tag_])
        final_token_3.append(dep_dict[token.dep_])
    #print('len :', len(final_token))
    return final_token_1, final_token_2, final_token_3

def get_tokens_plus(texts, max_seq_length=120):
    tokens1 = []
    tokens2 = []
    tokens3 = []
    for i in tqdm.tqdm(range(len(texts))):
        text = texts[i]
        tok1,tok2, tok3 = get_tokens(text)
        tok1 = add_padding(tok1, max_seq_length)
        tok2 = add_padding(tok2, max_seq_length)
        tok2 = add_padding(tok3, max_seq_length)
        tokens1.append(tok1)
        tokens2.append(tok2)
        tokens3.append(tok3)
        
    return np.array(tokens1), np.array(tokens2), np.array(tokens3)


#!/usr/bin/env python

import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
     name='nlp_preprocessing',  
     version='0.1.12',
     author="Ankur Singh",
     author_email="ankur310794@gmail.com",
     description="A Package for text preprocessing",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/Ankur3107/nlp_preprocessing",
     packages=['nlp_preprocessing'],
     install_requires=['scikit-learn','gensim','numpy','spacy','deepdish','tqdm'], 
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent"
     ]
 )